package com.gzlh.background.client;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.gzlh.bus.EventBus;
import com.gzlh.bus.SysConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.SystemObject;
import com.gzlh.config.dto.CaputreSetting;
import com.gzlh.config.hksdk.HCNetSDK;
import com.gzlh.config.hksdk.HkUtils;
import com.gzlh.config.hksdk.bo.HKCacheManager;
import com.gzlh.device.capture.brand.CaptureBrandType;
import com.gzlh.device.led.utils.LedOptions;
import com.gzlh.device.plc.event.PLCEvent;
import com.gzlh.entity.CommandBO;
import com.gzlh.utils.DeviceCache;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@ChannelHandler.Sharable
public class BackgroundClientHandler extends SimpleChannelInboundHandler<String> {

    private BackgroundClientNetty client;

    @Resource
    private EventBus eventBus;

    @Resource
    private HCNetSDK hcNetSDK;
    @Resource
    private HkUtils hkUtils;

    public BackgroundClientHandler(BackgroundClientNetty client) {
        this.client = client;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, String msg) throws Exception {
//        System.out.println("Client received: " + msg);
        if (!"back message".equalsIgnoreCase(msg)){
            log.info("后端 客户端收到的信息：{}",msg);
           handlerMsg(msg,ctx);
        }
    }
    TimedCache<String, String> CONTENT_CACHE = CacheUtil.newTimedCache(2000);
    void handlerMsg(String str,ChannelHandlerContext ctx){
        String channelId = ctx.channel().id().asShortText();
        if (!StrUtil.endWith(str, "}") && StrUtil.startWith(str, "{")) {
            CONTENT_CACHE.put(channelId, str);
            return;
        } else if (!StrUtil.endWith(str, "}") && !StrUtil.startWith(str, "{")) {
            String content = CONTENT_CACHE.get(channelId);
            content += str;
            CONTENT_CACHE.put(channelId, content);
            return;
        } else if (StrUtil.endWith(str, "}")) {
            String content = CONTENT_CACHE.get(channelId);
            if (StrUtil.isEmpty(content)) {
                content = "";
            }
            str = content + str;
            CONTENT_CACHE.remove(channelId);
        }
        CommandBO commandBO = JSONUtil.toBean(str, CommandBO.class);
        CommandBO.Command command = commandBO.getCommand();
        if (!StringUtils.isEmpty(command.getExtra())){
            LedOptions options = new LedOptions().setLine("04").setColor("03").setShowType("00");
            SystemObject.ledFactory.handler(SysConfig.serialSetting.getLed().getBrand())
                    .sendMsg(command.getExtra(),options);
        }
        if (StrUtil.isNotEmpty(command.getCommand())&&StrUtil.isNotEmpty(command.getModule())){
            eventBus.startEvent(command.getModule()+"."+command.getCommand());
        }
    }


    protected void sendHeartbeat(ChannelHandlerContext context) {
//        向后台发送设备信息json字符串
//        获取海康摄像头连接状态
        CaputreSetting caputreSetting=SysConfig.caputreSetting;
        if (caputreSetting.getBrand()== CaptureBrandType.HK_BRAND.getCode()){
            Map<String,Object> captrue = new HashMap<String,Object>();
            int userId = HKCacheManager.getUserId(ModuleEnum.CAPTURE_MODULE);
            boolean status = hkUtils.getStatus(hcNetSDK, userId);
            int loginV40=0;
            if (!status){
                short port= (short) caputreSetting.getPort();
                String ip=caputreSetting.getHost();
                loginV40 = hkUtils.Login_V40(ip, port, caputreSetting.getUsername(), caputreSetting.getPwd(), hcNetSDK);
                if (loginV40!=-1){
                    hkUtils.SetAlarm(hcNetSDK,loginV40);
                    HKCacheManager.add(ModuleEnum.CAPTURE_MODULE,loginV40);
                }
             }
            DeviceCache.changeDeviceStatus(ModuleEnum.CAPTURE_MODULE.getModuleZh(), loginV40!=-1?1:0);
        }

        String jsonStr = DeviceCache.getDeviceStatusJson();
        context.writeAndFlush(jsonStr);
    }


    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        // IdleStateHandler 所产生的 IdleStateEvent 的处理逻辑.
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent e = (IdleStateEvent) evt;
            switch (e.state()) {
                case READER_IDLE:
                    handleReaderIdle(ctx);
                    break;
                case WRITER_IDLE:
                    handleWriterIdle(ctx);
                    break;
                case ALL_IDLE:
                    handleAllIdle(ctx);
                    break;
                default:
                    break;
            }
        }
    }

    protected void handleReaderIdle(ChannelHandlerContext ctx) {
//        System.err.println("---READER_IDLE---");
    }

    protected void handleWriterIdle(ChannelHandlerContext ctx) {
        //写空闲时发送心跳包
        sendHeartbeat(ctx);

//        System.err.println("---WRITER_IDLE---");
    }

    protected void handleAllIdle(ChannelHandlerContext ctx) {
        System.err.println("---ALL_IDLE---");
//        sendHeartbeat(ctx);
    }


    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
       // ctx.close();
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.error("-------------后端服务器连接断开,{}-----------，进行重连", ctx.channel().remoteAddress());
//        System.err.println("---后端服务器连接断开，" + ctx.channel().remoteAddress() + " ---");
        client.connect();
    }

}
