package com.gzlh.background.client;

import com.gzlh.bus.SysConfig;
import com.gzlh.config.dto.ManagerSetting;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.timeout.IdleStateHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

@Configuration
@Slf4j
public class BackgroundClientNetty {


    @Autowired
    private BackgroundClientHandler backgroundClientHandler;
    Bootstrap bootstrap;
    Channel channel;


    @Bean("backgroundClient")
    public Bootstrap bootstrap() {
        ManagerSetting managerSetting= SysConfig.managerSetting;
        EventLoopGroup group = new NioEventLoopGroup();
        bootstrap = new Bootstrap();
        return bootstrap.group(group)
                .channel(NioSocketChannel.class)
                .remoteAddress(managerSetting.getHost(), managerSetting.getBeatPort())
                .handler(new ChannelInitializer<SocketChannel>() {
                    @Override
                    protected void initChannel(SocketChannel socketChannel) {
                        socketChannel.pipeline()
                                .addLast("decoder", new StringDecoder())
                                .addLast("encoder", new StringEncoder())
                                .addLast(new IdleStateHandler(0, 5, 0))
                                .addLast(backgroundClientHandler);
                    }
                });

    }

    @Bean("backgroundClientHandler")
    public BackgroundClientHandler backgroundClientHandler() {
        return new BackgroundClientHandler(this);
    }

    public void connect() {
        ChannelFuture future = bootstrap().connect();
        ManagerSetting managerSetting= SysConfig.managerSetting;
        future.addListener((ChannelFutureListener) future1 -> {
            if (future1.isSuccess()) {
                channel = future1.channel();
                log.info("后端服务器连接成功,{},{}",managerSetting.getHost(), managerSetting.getBeatPort());
            } else {
                log.error("-------------后端连接服务器失败,{},{}-----------，进行重连",managerSetting.getHost(), managerSetting.getBeatPort());
                future1.channel().eventLoop().schedule(this::connect, 5, TimeUnit.SECONDS);
            }
        });
        try {
            future.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public void send(String message) {
        if (channel != null && channel.isActive()) {
            channel.writeAndFlush(message);
        } else {
            log.error("未建立连接，无法发送消息");
        }
    }

    public void close() {
        if (channel != null) {
            channel.close();
        }
    }

}
