package com.gzlh.device.capture.task;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;

import com.gzlh.bus.SysConfig;
import com.gzlh.config.SystemObject;
import com.gzlh.config.cache.CarCache;
import com.gzlh.device.capture.event.CaptureEvent;

import com.gzlh.config.ModuleEnum;
import com.gzlh.bus.EventBus;
import com.gzlh.bus.EventDataManager;
import com.gzlh.config.task.Task;
import com.gzlh.device.led.utils.LedOptions;
import com.gzlh.device.weighbridge.handler.impl.CommonWeighbridgeHandler;
import com.gzlh.entity.ReqBO;
import com.gzlh.utils.DeviceCache;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.File;
import java.net.InetAddress;
import java.util.Date;

/***
 * 将本地的车牌图片、车牌号缓存到数据中心
 */
@Slf4j
public class HandlerCarImage extends Task {
    private String httpUrl;
    private String carNo;
    public HandlerCarImage(String id, long delayInMilliseconds, String httpUrl, String carNo) {
        super(id, delayInMilliseconds);
        this.httpUrl=httpUrl;
        this.carNo=carNo;
    }

    @SneakyThrows
    @Override
    public void run() {
        ReqBO reqBO = EventDataManager.getCacheData();
        //已经有车牌，又有车进来===>跟车
        if (reqBO!=null&&!StringUtils.isEmpty(reqBO.getCarNo())&& !StrUtil.equals(reqBO.getCarNo(),carNo)){
           //中断
            DeviceCache.setInterrupt(true);
            LedOptions options = new LedOptions().setShowType("00");
            SystemObject.ledFactory.handler(SysConfig.serialSetting.getLed().getBrand())
                    .sendMsg("请勿跟车，请退出重进", options.setLine("04").setColor("01"));
            return;
        }
        CommonWeighbridgeHandler.hsCar=true;
        EventBus eventBus=SpringUtil.getBean(EventBus.class);
        //向总线发送抓拍到车牌事件
        if (SysConfig.channelSetting.getChannelType() ==0){
            EventDataManager.cacheData("carImg",httpUrl);
            EventDataManager.cacheData("carNo",carNo);
            EventDataManager.cacheData("gatherTime",new Date());
            eventBus.startEvent(ModuleEnum.CAPTURE_MODULE.getModuleEn()+"."+ CaptureEvent.CAPTURE_CAR_NO);
        }else{
            CarCache.handlerCarCache(carNo,httpUrl);
            eventBus.startEvent(ModuleEnum.CAPTURE_MODULE.getModuleEn()+"."+ CaptureEvent.CAPTURE_ADMIN);
        }
    }
}
