package com.gzlh.device.electron.client;

import cn.hutool.core.util.StrUtil;
import com.gzlh.bus.SysConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.SystemObject;
import com.gzlh.device.electron.action.IElectronAction;
import com.gzlh.utils.DeviceCache;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@ChannelHandler.Sharable
public class ElectronClientHandler extends SimpleChannelInboundHandler<String> {

    private ElectronNettyConfig electronNettyConfig;

    public ElectronClientHandler(ElectronNettyConfig electronNettyConfig) {
        this.electronNettyConfig = electronNettyConfig;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, String msg) throws Exception {
        DeviceCache.changeDeviceStatus(ModuleEnum.ELECTRON_MODULE.getModuleZh(), 1);
        if (StrUtil.isEmpty(msg)){
            return;
        }
        if (StrUtil.sub(msg,4,6).equalsIgnoreCase("02")){
            return;
        }
        log.info("电子车牌 客户端收到消息：" + msg);
        SystemObject.electronFactory.handler(SysConfig.serialSetting.getElectron().getBrand())
                .handlerMsg(msg.toUpperCase());
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.error("电子车牌 客户端 连接断开，进行重连");
        DeviceCache.changeDeviceStatus(ModuleEnum.ELECTRON_MODULE.getModuleZh(), 0);
        electronNettyConfig.connect();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        ctx.close();
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        // IdleStateHandler 所产生的 IdleStateEvent 的处理逻辑.
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent e = (IdleStateEvent) evt;
            switch (e.state()) {
                case READER_IDLE:
                    handleReaderIdle(ctx);
                    break;
                case WRITER_IDLE:
                    handleWriterIdle(ctx);
                    break;
                case ALL_IDLE:
                    handleAllIdle(ctx);
                    break;
                default:
                    break;
            }
        }
    }
    protected void handleReaderIdle(ChannelHandlerContext ctx) {
//        System.err.println("---READER_IDLE---");
    }

    protected void handleWriterIdle(ChannelHandlerContext ctx) {
        //写空闲时发送读取版本信息指令
        SystemObject.electronFactory.handler(SysConfig.serialSetting.getElectron().getBrand())
                .handlerAction(IElectronAction.VERSION);
//        System.err.println("---WRITER_IDLE---");
    }

    protected void handleAllIdle(ChannelHandlerContext ctx) {
        System.err.println("---ALL_IDLE---");
//        sendHeartbeat(ctx);
    }
}

