package com.gzlh.device.electron.handler.impl;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.gzlh.bus.EventDataManager;
import com.gzlh.device.electron.action.IElectronAction;
import com.gzlh.device.electron.brand.ElectronBrandType;
import com.gzlh.device.electron.client.ElectronNettyConfig;
import com.gzlh.device.electron.handler.IElectronHandler;
import com.gzlh.device.electron.utils.CheckNumUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;

/**
 * 瑞德电子车牌处理类
 */
@Component
@Slf4j
public class ElectronHandlerRuiDe implements IElectronHandler {
    private final String CALL_BACK_HEADER="F0";

    private final static List<String> ERI_CACHE = new CopyOnWriteArrayList<>();

    @Resource
    private ElectronNettyConfig electronNettyConfig;
    @Override
    public ElectronBrandType brandType() {
        return ElectronBrandType.RUI_BRAND;
    }

    @Override
    public void handlerAction(String action) {
        if (action.equals(IElectronAction.START_READ_CARD)){
            readERI();
        }else if (action.equals(IElectronAction.VERSION)) {
            getVersion();
        }
    }

    @Override
    public void sendMsg(String msg) {
        electronNettyConfig.send(msg);
    }

    @Override
    public void handlerMsg(String msg) {
        int len=msg.length();
        //引导码
        String bootCode=StrUtil.sub(msg,0,2);
        if (StrUtil.equals(bootCode,CALL_BACK_HEADER)){
            //长度
            String lenHex=StrUtil.sub(msg,2,4);
            //命令Command
            String command=StrUtil.sub(msg,4,6);
            //return data，返回的数据
            String data= StrUtil.sub(msg,0,len-2);
            //校验码
           String checkNum= StrUtil.sub(msg,len-2,len);
           //计算出来的校验码
           String num=  CheckNumUtils.calculateChecksum(data);
           if (!StrUtil.equals(checkNum,num)){
               return;
           }
            String returnData = StrUtil.sub(data,6, len);
            int count = Integer.parseInt(StrUtil.sub(returnData,0, 2));
            String idStr = returnData.substring(2);
            String[] idArray = new String[count];
           for (int i=0;i<count;i++){
//               id长度为16
               ERI_CACHE.add(StrUtil.sub(idStr,i*16, (i+1)*16));
           }
        }
    }


    public void readERI(){
        ERI_CACHE.clear();
        int i=0;
        while (i<10){
            i++;
//            发送读取电子车牌的指令
            sendMsg("4002FEC0");
            ThreadUtil.sleep(500);
        }
        if (ERI_CACHE.size()<1){
            return;
        }
        Map<String, Long> collect = ERI_CACHE.stream().collect(Collectors.groupingBy(v -> v, Collectors.counting()));
        Map<String, Long> maxValue = getMaxValue(collect);
//        判断数据是否只有一个
//        if (maxValue.size()>1){
//            readERI();
//        }else {
            for (String eri:maxValue.keySet()){
                EventDataManager.cacheData("eri",eri);
            }
//        }
        ERI_CACHE.clear();

    }

    public static Map<String ,Long> getMaxValue(Map<String, Long> map){
        if(map == null){
            return new HashMap<>();
        }
        Map<String,Long> maxMap = new HashMap<>();
        int length =map.size();
        Collection<Long> c = map.values();
        Object[] obj = c.toArray();
        Arrays.sort(obj);
        for(Map.Entry<String,Long> entry : map.entrySet()){
            if(entry.getValue() == obj[length-1]){
                maxMap.put(entry.getKey(),entry.getValue());
            }
        }
        return maxMap;
    }

    public void getVersion(){
//            发送读取版本信息的指令
            sendMsg("400202BC");
    }
}
