package com.gzlh.device.weighbridge.client;

import cn.hutool.extra.spring.SpringUtil;
import com.gzlh.bus.SysConfig;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.device.weighbridge.factory.WeighbridgeFactory;
import com.gzlh.utils.DeviceCache;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@ChannelHandler.Sharable
public class WeighbridgeClientHandler extends SimpleChannelInboundHandler<String> {

    private WeighbridgeNettyConfig nettyConfig;

    public WeighbridgeClientHandler(WeighbridgeNettyConfig nettyConfig) {
        this.nettyConfig = nettyConfig;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, String msg) throws Exception {
        try {
            SerialSetting.Weighbridge weighbridge = SysConfig.serialSetting.getWeighbridge();
            DeviceCache.changeDeviceStatus(ModuleEnum.WEIGHBRIDGE_MODULE.getModuleZh(),1);
            WeighbridgeFactory weighbridgeFactory = SpringUtil.getBean(WeighbridgeFactory.class);
            weighbridgeFactory.handler(weighbridge.getBrand()).handlerMsg(msg);
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.error("地磅 连接断开，进行重连");
        DeviceCache.changeDeviceStatus(ModuleEnum.WEIGHBRIDGE_MODULE.getModuleZh(),0);
        nettyConfig.connect();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        log.error("weight error:{}",cause.getMessage());
    }


}

