package com.gzlh.device.weighbridge.client;

import com.gzlh.bus.SysConfig;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.config.netty.NettyDecoder;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.nio.charset.StandardCharsets;
import java.util.concurrent.TimeUnit;

@Configuration
@Slf4j
public class WeighbridgeNettyConfig {


    @Autowired
    private WeighbridgeClientHandler nettyClientHandler;

    private Channel channel;

    @Bean("weighbridge")
    public Bootstrap bootstrap() {
        SerialSetting serialSetting = SysConfig.serialSetting;
        String host = serialSetting.getHost();
        SerialSetting.Weighbridge weighbridge = serialSetting.getWeighbridge();
        int port = weighbridge.getPort();
        if (weighbridge.getEnable()) {
            log.info("初始化 地磅 ======================:{},{}", host, port);
        }
        EventLoopGroup group = new NioEventLoopGroup();
        return new Bootstrap()
                .group(group)
                .channel(NioSocketChannel.class)
                .remoteAddress(host, port)
                .option(ChannelOption.SO_KEEPALIVE, true)
                .handler(new ChannelInitializer<SocketChannel>() {
                    @Override
                    protected void initChannel(SocketChannel ch) {
                        try {
                            ChannelPipeline pipeline = ch.pipeline();
                            pipeline.addLast("decoder", new StringDecoder(StandardCharsets.UTF_8));
                            pipeline.addLast("encoder", new StringEncoder(StandardCharsets.UTF_8));

                            pipeline.addLast("handler", nettyClientHandler);
                        } catch (Exception e) {
                            log.info("error connect:{}", e.getMessage());
                        }
                    }
                });
    }

    @Bean
    public WeighbridgeClientHandler nettyClientHandler() {
        return new WeighbridgeClientHandler(this);
    }

    public void connect() {
        SerialSetting serialSetting = SysConfig.serialSetting;
        String host = serialSetting.getHost();
        SerialSetting.Weighbridge weighbridge = serialSetting.getWeighbridge();
        int port = weighbridge.getPort();
        ChannelFuture future = bootstrap().connect();
        future.addListener((ChannelFutureListener) future1 -> {
            if (future1.isSuccess()) {
                channel = future1.channel();
                log.info("地磅 连接服务器成功,{},{}", host, port);
            } else {
                log.error("-------------地磅 连接服务器失败-----------，进行重连");
                future1.channel().eventLoop().schedule(this::connect, 5, TimeUnit.SECONDS);
            }
        });
        try {
            future.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public void send(String message) {
        if (channel != null && channel.isActive()) {
            channel.writeAndFlush(message);
        } else {
            log.error("未建立连接，无法发送消息");
        }
    }

    public void close() {
        if (channel != null) {
            channel.close();
        }
    }
}

