package com.gzlh.device.weighbridge.handler.impl;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.json.JSONUtil;
import cn.hutool.log.StaticLog;
import com.gzlh.bus.ChannelCacheManager;
import com.gzlh.bus.SysConfig;
import com.gzlh.bus.EventDataManager;
import com.gzlh.bus.EventBus;
import com.gzlh.config.SystemObject;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.device.led.utils.LedOptions;
import com.gzlh.device.weighbridge.event.WeighbridgeEvent;
import com.gzlh.utils.DeviceCache;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;
import java.util.stream.Stream;
@Slf4j
public class CommonWeighbridgeHandler {


    private final static List<Integer> WEIGHT_CACHE = new CopyOnWriteArrayList<>();


    public static boolean isStart = false;
    public static boolean hsCar = false;


    /**
     * 添加地磅数据到缓存
     *
     * @param weight 地磅数
     */
    public static void cacheWeight(int weight) {
        if (!isStart) {
            WEIGHT_CACHE.clear();
            return;
        }
        WEIGHT_CACHE.add(weight);
    }

    /**
     * 开始读取地磅数据
     */
    public static void startToRead() {
        //不启用
        if (!ChannelCacheManager.stateEnable(SysConfig.channelSetting.getChannelCode())){
            return;
        }
        isStart = true;
        ThreadUtil.execute(new WatchThread());
    }

    /**
     * 监控地磅缓存是否读取到了数据
     * 监控2秒内的地磅数
     */
    static class WatchThread implements Runnable {
        private int time = 4000;

        @Override
        public void run() {
            int averaWeight = 0;
            String deviceStatus = null;
            SerialSetting serialSetting = SysConfig.serialSetting;
            SerialSetting.PlcDTO.StatusDTO status = serialSetting.getPlc().getStatus();
            EventBus eventBus = SpringUtil.getBean(EventBus.class);
            while (time > 0) {
                if (!isStart) {
                    return;
                }
                time = time - 1000;
//                检查前后红外和雷达的触发情况
  //              deviceStatus = new StringBuffer(DeviceCache.getPlcStatus()).reverse().toString();
                ThreadUtil.sleep(1000);
            }
            isStart = false;
            averaWeight = handlerWeight();
            log.info("----------weight---------:{}", averaWeight);
            if (averaWeight == 0 || serialSetting.getWeighbridge().getMinKg() > averaWeight) {
                //todo read timeout
                //读取超时
                DeviceCache.setInterrupt(true);
                LedOptions options = new LedOptions().setLine("03").setColor("04").setShowType("0B");
                SystemObject.ledFactory.handler(SysConfig.serialSetting.getLed().getBrand())
                        .sendMsg("称重异常", options);
                SystemObject.ledFactory.handler(SysConfig.serialSetting.getLed().getBrand())
                        .sendMsg("请退出通道", options.setLine("04").setColor("01"));
                return;
            }
            EventDataManager.cacheData("weight", averaWeight);
            EventDataManager.cacheData("gatherTime", new Date());
        }

        private int handlerWeight() {
            List<Integer>list=WEIGHT_CACHE;
            log.info("w:{}", JSONUtil.toJsonStr(list));
            if (list.isEmpty()){
                return 0;
            }
            Collections.sort(list);
            int size=list.size();
            int index=( size*3)/4;
           return list.get(index);
        }
    }

    public static void main(String[] args) {
//        Integer[]array=new Integer[]{50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50080,50080,50080,50080,50080,50080,50080,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50120,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50100,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120,50120};
        Integer[]array=new Integer[]        {2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020,2020};
        List<Integer>list= Stream.of(array).collect(Collectors.toList());
        Collections.sort(list);
        int size=list.size();
        int index=( size*3)/4;
        System.out.println(index);
        System.out.println(list.get(index));
    }
}
