package com.gzlh.device.weighbridge.handler.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.gzlh.bus.EventDataManager;
import com.gzlh.bus.SysConfig;
import com.gzlh.config.hksdk.AlarmParseService;
import com.gzlh.config.hksdk.MessageCallBack;
import com.gzlh.device.weighbridge.action.WeighbridgeCommonAction;
import com.gzlh.device.weighbridge.brand.WeighbridgeBrandType;
import com.gzlh.device.weighbridge.handler.IWeighbridgeHandler;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Map;

/**
 * 宁波柯力地磅处理器
 */
@Service
@Slf4j
public class WeighbridgeHandlerKeLi implements IWeighbridgeHandler {

    @Override
    public WeighbridgeBrandType brandType() {
        return WeighbridgeBrandType.NINGBO_KELI;
    }


    @Override
    public void handlerMsg(String msg) {
        msg = StrUtil.removeAll(msg, "\u0002");
        msg = StrUtil.removeAll(msg, "\r");
        msg = msg.replaceFirst("^0*", "");
//        log.info(WeighbridgeBrandType.NINGBO_KELI.getBrand() + "地磅:{}", msg);
        if (StrUtil.length(msg)<8){
            return;
        }
        msg=msg.substring(3);
        Integer weight = convertMsgToWeight(msg);
//        log.info(WeighbridgeBrandType.NINGBO_KELI.getBrand() + "地磅 weight:{}", weight);
        if (weight==0){
            CommonWeighbridgeHandler.hsCar=false;
        }
        Integer minKg = SysConfig.serialSetting.getWeighbridge().getMinKg();
        CommonWeighbridgeHandler.hsCar=true;
        if (minKg > weight) {
            CommonWeighbridgeHandler.hsCar=false;
            return;
        }
//        log.info(WeighbridgeBrandType.NINGBO_KELI.getBrand() + "地磅:{}", weight);
        // 将字符串转换成字节数组类型
        // int weight = Integer.parseInt(new String(validData)) / 10;
        CommonWeighbridgeHandler.cacheWeight(weight);
    }

    @Override
    public void handlerAction(String actionCommand) {
        actionCommand = actionCommand.toUpperCase();
        if (StrUtil.equals(actionCommand, WeighbridgeCommonAction.START_READ)) {
            //开始读取地磅数据
            CommonWeighbridgeHandler.startToRead();
        } else if (StrUtil.equals(actionCommand, WeighbridgeCommonAction.CLEAN)) {
            EventDataManager.cleanData();
            AlarmParseService.CACHE_MAP.clear();
            CommonWeighbridgeHandler.hsCar=false;
        }
    }


    private static Integer convertMsgToWeight(String msg) {
        //粘包的情况下，截取第一个数据包数据包;
        msg = msg.substring(0, 12);
        //重量包
//        msg = msg.substring(3);
        //替换前面的所有"0"
        msg = msg.replaceFirst("^0*", "");
        int len = msg.length();
        String numStr = "0";
        for (int i = 0; i < len; i++) {
            char bit = msg.charAt(i);
            if (!"0".equals(bit + "")) {
                numStr = msg.substring(i);
                break;
            }
        }
        Long we=Long.parseLong(numStr) / 1000000;
        return Integer.parseInt(we+"");
    }

    public static void main(String[] args) {
        String str = "0090 00010000000090 000100000000";
        str = str.replaceFirst("^0*", "");
        System.out.println(str);
        str=str.substring(3);
//        log.info(WeighbridgeBrandType.NINGBO_KELI.getBrand() + "地磅:{}", msg);
        // 去掉开头和结尾的特殊字符
        Integer weight = convertMsgToWeight(str);
        System.out.println(weight);
    }

}
