package com.gzlh.startup;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.gzlh.background.client.BackgroundClientNetty;
import com.gzlh.bus.SysConfig;
import com.gzlh.bus.task.CheckInitTask;
import com.gzlh.config.dto.CaputreSetting;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.SystemObject;
import com.gzlh.bus.EventBus;
import com.gzlh.config.hksdk.HCNetSDK;
import com.gzlh.config.hksdk.HkUtils;
import com.gzlh.config.hksdk.bo.HKCacheManager;
import com.gzlh.config.task.TaskService;
import com.gzlh.device.capture.brand.CaptureBrandType;
import com.gzlh.device.electron.client.ElectronNettyConfig;
import com.gzlh.device.led.client.LedNettyConfig;
import com.gzlh.device.plc.client.PlcNettyConfig;
import com.gzlh.device.plc.event.PLCEvent;
import com.gzlh.device.plc.job.SearchJob;
import com.gzlh.device.weighbridge.client.WeighbridgeNettyConfig;

import com.gzlh.hx.api.IceSdk;
import com.gzlh.hx.callback.PlateCallback;
import com.gzlh.hx.handler.InitHxSDK;
import com.sun.jna.Pointer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Component
@Slf4j
public class StartupRunner implements CommandLineRunner {

    @Resource
    private WeighbridgeNettyConfig weighbridgeNettyConfig;

    @Resource
    private LedNettyConfig ledNettyConfig;
    @Resource
    private PlcNettyConfig plcNettyConfig;
    @Resource
    private ElectronNettyConfig electronNettyConfig;
    @Resource
    private TaskService taskService;


    @Resource
    private BackgroundClientNetty backgroundClientNetty;

    @Resource
    private HkUtils hkUtils;
    @Resource
    private HCNetSDK hcNetSDK;

    @Override
    public void run(String... args) throws Exception {

        SerialSetting serialSetting = SysConfig.serialSetting;
        if (serialSetting.getWeighbridge().getEnable()) {
            ThreadUtil.execute(() -> weighbridgeNettyConfig.connect());
        }
        if (serialSetting.getLed().getEnable()) {
            ThreadUtil.execute(() -> ledNettyConfig.connect());
        }
        if (serialSetting.getPlc().getEnable()) {
            ThreadUtil.execute(() -> plcNettyConfig.connect());
            ThreadUtil.execute(new SearchJob());
        }
        if (serialSetting.getElectron().getEnable()) {
            ThreadUtil.execute(() -> electronNettyConfig.connect());
        }
        if (SysConfig.managerSetting.isEnable()) {
            ThreadUtil.execute(() -> backgroundClientNetty.connect());
        }
        CaputreSetting caputreSetting = SysConfig.caputreSetting;
        if (caputreSetting.isEnable()) {
            if (caputreSetting.getBrand() == CaptureBrandType.HK_BRAND.getCode()) {
                short port = (short) caputreSetting.getPort();
                String ip = caputreSetting.getHost();
                int userId = hkUtils.Login_V40(ip, port, caputreSetting.getUsername(), caputreSetting.getPwd(), hcNetSDK);
                if (userId != -1) {
                    log.info("登录海康抓拍设备:{},{}", ip, userId);
                    hkUtils.SetAlarm(hcNetSDK, userId);
                    HKCacheManager.add(ModuleEnum.CAPTURE_MODULE, userId);
                } else {
                    log.info("登录海康抓拍设备失败:{},{}", hcNetSDK.NET_DVR_GetLastError(), userId);
                }
                taskService.addTask(new CheckInitTask(RandomUtil.randomNumbers(16), 10000, 0));

            } else if (caputreSetting.getBrand() == CaptureBrandType.HX_BRAND.getCode()) {
                ThreadUtil.execute(new InitHxSDK());
            }
        }
    }


}
